#!/bin/sh
#----------------------------------------------------------------------------------------
# /var/install/config.d/owswitch - configuration generator script for owswitch service
#
# Copyright (c) 2016-2016 The Eisfair Team, team(at)eisfair(dot)org
#
# Creation:    2016-06-13 jed
# Last Update: $Id: owswitch.sh 42022 2016-06-25 13:50:58Z jed $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#----------------------------------------------------------------------------------------

# read eislib
. /var/install/include/eislib
. /var/install/include/jedlib

# activate debug output
#debug_trace=true
if ${debug_trace:-false}
then
    exec 2> /tmp/$(basename ${0})-trace-$$.log
    set -x
    ask_debug=true
    export ask_debug
fi

module_name='owswitch'
pgmname=`basename $0`

configfile=/etc/config.d/${module_name}
packagefile=/var/install/packages/${module_name}
generate_aliases=/etc/owfs-aliases.${module_name}
nagiospackagefile=/var/install/packages/nagios
nagiosressoucefile=/etc/nagios/resource.cfg
owfsfile=/etc/config.d/owfs
owswitchrunpath=/var/lib/${module_name}
owswitchrunfile=${owswitchrunpath}/${module_name}.run
owswitchcmdfile=/var/lib/owswitch/owswitch-cmd  # sensor-id, pio-port, etc. will be added

### other parameters ###
package_version="v`grep "<version>" ${packagefile} | sed 's#<version>\(.*\)</version>#\1#'`"

### load configuration ###
. ${configfile}
chmod 600 ${configfile}

# !!!!! check_owswitch_status kopieren !!!!!

#----------------------------------------------------------------------------------------
# check if owfs has been enabled
#
# input : [--quiet] - be quiet, suppress all output
#
# return: 0 - owfs package enabled
#         1 - owfs package disabled
#----------------------------------------------------------------------------------------
check_installed_owfs ()
{
    retval=1

    if [ -f ${owfsfile} ]
    then
        # owfs installed
        . ${owfsfile}

        if [ "${START_OWFS}" = 'yes' -a "${OWFS_SERVER_OWFS}" = 'yes' ]
        then
            # owfs activated
            if [ "${1}" != '--quiet' ]
            then
                mecho "OWFS server has been enabled ..."
            fi
            retval=0
        else
            # owfs deactivated
            if [ "${1}" != '--quiet' ]
            then
                mecho --warn "OWFS server has been disabled ..."
            fi
        fi
    fi

    return ${retval}
}

#----------------------------------------------------------------------------------------
# check sensor availability
#----------------------------------------------------------------------------------------
check_sensor_status ()
{
    mecho "checking sensor stati ..."

    # check if configured sensors are reachable
    _css_idx=1
    _css_sensors=':'
    _css_warning=0
    while [ ${_css_idx} -le ${OWSWITCH_N} ]
    do
        eval _css_sensorid='$OWSWITCH_'${_css_idx}'_ID'

        owread ${_css_sensorid}/type >/dev/null 2>/dev/null

        if [ $? -ne 0 ]
        then
            _css_warning=1
            mecho --warn "- sensor '${_css_sensorid}' is currently unreachable!"
        fi

        _css_sensors="${_css_sensors}${_css_sensorid}:"

        _css_idx=`expr ${_css_idx} + 1`
    done

    if [ ${_css_warning} -eq 0 ]
    then
        echo "- all sensors reachable."
    fi

    # check if available sensors have been configured
    eval `grep "OWFS_SERVER_OWFS_MOUNT" ${owfsfile}`

    _css_warning=0
    for SENSORADDR in `find ${OWFS_SERVER_OWFS_MOUNT} -maxdepth 2 -name "address" | sed "s#^${OWFS_SERVER_OWFS_MOUNT}/##"`
    do
        # '3A698402010800A0' -> '3A.698402010800'
        _css_sensorid=`owread ${SENSORADDR} | grep "^3A"`

        if [ ! -z "${_css_sensorid}" ]
        then
            _css_sensorid="`echo "${_css_sensorid}" | sed 's/^\(..\)\(.*\)..$/\1.\2/'`"

            echo "${_css_sensors}" | grep -q ":${_css_sensorid}:"

            if [ $? -ne 0 ]
            then
                mecho --warn "- sensor '${_css_sensorid}' not configured!"
                _css_sensors="${_css_sensors}:${_css_sensorid}"
            fi
        fi
    done

    if [ ${_css_warning} -eq 0 ]
    then
        echo "- all available sensors configured."
    fi
}

#----------------------------------------------------------------------------------------
# generate aliases file
#----------------------------------------------------------------------------------------
create_owfs_aliases ()
{
    mecho "creating `basename ${generate_aliases}` ..."

    {
        #-------------------------------------------------------------------------
        print_short_header "${generate_aliases}" "${pgmname}" "${module_name}" "${package_version}"
        #-------------------------------------------------------------------------

        idx=1
        alias_exists=0
        while [ ${idx} -le ${OWSWITCH_N} ]
        do
            eval active='$OWSWITCH_'${idx}'_ACTIVE'

            if [ "${active}" = "yes" ]
            then
                alias_exists=1

                eval sensor_id='$OWSWITCH_'${idx}'_ID'
                eval sensor_name='$OWSWITCH_'${idx}'_NAME'

                sensor_name=`echo "${sensor_name}" | tr -s ' ' '_'`

                if [ "${sensor_id}" != '' ]
                then
                    echo "${sensor_id} = ${sensor_name}"
                fi
            fi

            idx=`expr ${idx} + 1`
        done
    } > ${generate_aliases}

    if [ ${alias_exists} -eq 1 ]
    then
        # update aliases
        /var/install/config.d/owfs.sh --alias
    else
        # empty file delete it
        rm -f ${generate_aliases}
        /var/install/config.d/owfs.sh --alias
    fi
}

#----------------------------------------------------------------------------------------
# create command files
#----------------------------------------------------------------------------------------
create_cmd_files ()
{
    mecho "creating command files ..."

    if [ ! -f ${owswitchrunpath} ]
    then
        mkdir -p ${owswitchrunpath}
    fi

    if [ "${OWFS_INSTALLED}" = 'yes' ]
    then
        # remove previously created command files
        rm -f ${owswitchcmdfile}-3A\.[0-9A-Z]*-[0-9]*-on
        rm -f ${owswitchcmdfile}-3A\.[0-9A-Z]*-[0-9]*-on.run
        rm -f ${owswitchcmdfile}-3A\.[0-9A-Z]*-[0-9]*-off
        rm -f ${owswitchcmdfile}-3A\.[0-9A-Z]*-[0-9]*-off.run

        # read parameters
        idx=1
        while [ ${idx} -le ${OWSWITCH_N} ]
        do
            # check for active sensor
            eval ow_active='$OWSWITCH_'${idx}'_ACTIVE'

            if [ "${ow_active}" = 'yes' ]
            then
                # read sensor id
                # sensor id, e.g. 3A.3CF813000000
                eval ow_id='$OWSWITCH_'${idx}'_ID'

                if [ "${ow_id}" != '' ]
                then
                    jdx=1
                    while [ ${jdx} -le ${OWSWITCH_1_PIO_N} ]
                    do
                        # pio function: input, output or none
                        eval pio_func='$OWSWITCH_'${idx}'_PIO_'${jdx}'_FUNC'

                        case ${pio_func} in
                            none )
                                ;;
                            input|input-uncached|output|output-uncached )
                                # add sensor id only once
                                eval pio_on_active='$OWSWITCH_'${idx}'_PIO_'${jdx}'_ON_EXEC_ACTIVE'
                                eval pio_off_active='$OWSWITCH_'${idx}'_PIO_'${jdx}'_OFF_EXEC_ACTIVE'
                                eval pio_on_exec='$OWSWITCH_'${idx}'_PIO_'${jdx}'_ON_EXEC'
                                eval pio_off_exec='$OWSWITCH_'${idx}'_PIO_'${jdx}'_OFF_EXEC'

                                if [ "${pio_on_active}" = 'yes' -a "${pio_on_exec}" != '' ]
                                then
                                    echo "${pio_on_exec}" > ${owswitchcmdfile}-${ow_id}-${jdx}-on
                                fi

                                if [ "${pio_off_active}" = 'yes' -a "${pio_off_exec}" != '' ]
                                then
                                    echo "${pio_off_exec}" > ${owswitchcmdfile}-${ow_id}-${jdx}-off
                                fi
                                ;;
                            * )
                                # invalid choice
                                ;;
                        esac

                        jdx=`expr ${jdx} + 1`
                    done
                else
                    echo "error: OWSWITCH_${idx}_ID not set!"
                fi
            fi

            idx=`expr ${idx} + 1`
        done
    fi
}

#----------------------------------------------------------------------------------------
# copy nagios plugin to plugin directory
#----------------------------------------------------------------------------------------
copy_nagios_plugin ()
{
    if [ -f ${nagiospackagefile} -a -f ${nagiosressoucefile} ]
    then
        # $USER1$=/usr/local/nagios/plugins
        plugin_path=`grep '^$USER1$=' ${nagiosressoucefile} | cut -d'=' -f2`
        plugin_destfile=${plugin_path}/check_owswitch_status

        if [ -d ${plugin_path} ]
        then
            mecho "copying owSwitch nagios plugin to plugins folder ..."
            cp /usr/share/doc/${module_name}/check_owswitch_status ${plugin_destfile}
        fi
    fi
}

#----------------------------------------------------------------------------------------
# set access rights and ownership
#----------------------------------------------------------------------------------------
set_access_rights ()
{
    mecho "setting access rights and ownership ..."

    if [ -f ${generate_aliases} ]
    then
        chmod -R 0644  ${generate_aliases}
        chown -R root: ${generate_aliases}
    fi

    if [ -f ${owswitchrunfile} ]
    then
        chmod 0644  ${owswitchrunfile}
        chown root: ${owswitchrunfile}
    fi

    if [ -f ${nagiospackagefile} -a -f ${nagiosressoucefile} ]
    then
        # $USER1$=/usr/local/nagios/plugins
        plugin_path=`grep '^$USER1$=' ${nagiosressoucefile} | cut -d'=' -f2`
        plugin_destfile=${plugin_path}/check_owswitch_status

        if [ -d ${plugin_path} ]
        then
            chown nagios:nagios ${plugin_destfile}
            chmod 0550 ${plugin_destfile}
        fi
    fi
}

#========================================================================================
# main
#========================================================================================

case "$1" in
    setrights )
        set_access_rights
        ;;

    * )
        # generate all configuration files
        mecho "version: ${package_version}"

        if [ "${START_OWSWITCH}" = 'yes' ]
        then
            OWFS_INSTALLED='no'
            if check_installed_owfs
            then
                # installed
                OWFS_INSTALLED='yes'

                # read owfs configuration
                if [ -f ${configfile} ]
                then
                    if [ ! -f ${owswitchrunpath} ]
                    then
                        mkdir -p ${owswitchrunpath}
                    fi

                    cp ${configfile} ${owswitchrunfile}
                fi

                check_sensor_status
                create_cmd_files
                copy_nagios_plugin
                create_owfs_aliases
                set_access_rights
            fi
        else
            if [ -f ${create_owfs_aliases} ]
            then
                rm -f ${create_owfs_aliases}
                /var/install/config.d/owfs.sh --alias
            fi
        fi

        mecho "finished."
        ;;
esac

#========================================================================================
# End
#========================================================================================
exit 0
